import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/foundation.dart';

abstract class ReadOnlyDatabaseItem<T> {
  String id;
}

abstract class UpdateableDatabaseItem<T> extends ReadOnlyDatabaseItem<T> {
  T copy();
  Map<String, dynamic> toMap();

  bool equals(UpdateableDatabaseItem<T> item) {
    return mapEquals(item.toMap(), toMap());
  }
}

class ModelList<T> {
  final DocumentSnapshot snapshot;
  final String fieldName;
  final T Function(DocumentSnapshot snapshot) objectFromSnapshot;

  List<DocumentReference> references;

  ModelList(this.snapshot, this.fieldName, this.objectFromSnapshot);

  List<DocumentReference> getReferences() {
    return references = snapshot.data()[fieldName] == null
        ? <DocumentReference>[]
        : List.from(snapshot.data()[fieldName]);
  }

  List<Future<T>> getObjectList() {
    var list = <Future<T>>[];
    for (var ref in (references ?? getReferences())) {
      list.add(ref.get()?.then(objectFromSnapshot));
    }
    return list;
  }
}
