import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class NavigatorWidget extends StatefulWidget {
  final Widget menuWidget;

  const NavigatorWidget({
    Key key,
    this.menuWidget,
  }) : super(key: key);

  @override
  _NavigatorWidgetState createState() => _NavigatorWidgetState();
}

class _NavigatorWidgetState extends State<NavigatorWidget> {
  final RouteObserver<PageRoute> routeObserver = RouteObserver<PageRoute>();

  bool showRightSide = false;

  @override
  Widget build(BuildContext context) {
    SplitNavigator.mainContext = context;

    return Scaffold(
      body: Row(
        children: <Widget>[
          widget.menuWidget ?? Container(),
          Expanded(
            child: Navigator(
              key: SplitNavigator.leftNavigatorKey,
              onGenerateRoute: (settings) => MaterialPageRoute(
                builder: (context) {
                  return Container();
                },
              ),
            ),
          ),
          Container(
            width: 20,
            height: MediaQuery.of(context).size.height,
            color: CleanAndroidDesignDefaultColors.background,
          ),
          AnimatedContainer(
            duration: Duration(milliseconds: 150),
            width: showRightSide ? (MediaQuery.of(context).size.width / 2) : 0,
            child: Navigator(
              key: SplitNavigator.rightNavigatorKey,
              onGenerateRoute: (settings) => NoPageRoute(this, routeObserver),
              observers: [routeObserver],
            ),
          ),
        ],
      ),
    );
  }

  void setShowRightSide(bool showRightSide) {
    setState(() => this.showRightSide = showRightSide);
  }
}

class NavigatorListener with RouteAware {
  int n = -1;

  final _NavigatorWidgetState _navigatorWidgetState;

  NavigatorListener(this._navigatorWidgetState);

  @override
  void didPush() {
    super.didPush();
    n++;
    if (n == 1) _navigatorWidgetState.setShowRightSide(true);
  }

  @override
  void didPushNext() {
    super.didPushNext();
    n++;
    if (n == 1) _navigatorWidgetState.setShowRightSide(true);
  }

  @override
  void didPop() {
    super.didPop();
    n--;
    if (n == 0) _navigatorWidgetState.setShowRightSide(false);
  }

  @override
  void didPopNext() {
    super.didPopNext();
    n--;
    if (n == 0) _navigatorWidgetState.setShowRightSide(false);
  }
}

class NoPageRoute extends MaterialPageRoute {
  NoPageRoute(_NavigatorWidgetState _navigatorWidgetState,
      RouteObserver<PageRoute> routeObserver)
      : super(
          builder: (context) {
            routeObserver.subscribe(NavigatorListener(_navigatorWidgetState),
                ModalRoute.of(context));
            return Center(
              child: Text("NoRoute0"),
            );
          },
        );
}
