import 'dart:math';

import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class ScaffoldPageWidget<T> extends StatelessWidget {
  final Widget child;
  final ScaffoldPageIcon pageIcon;
  final VoidCallback onShowPage;
  final Validator<T> validator;
  final bool Function(T item) isValid;

  const ScaffoldPageWidget({
    Key key,
    this.pageIcon,
    this.child,
    this.onShowPage,
    this.validator,
    this.isValid,
  })  : assert(validator == null || isValid != null),
        assert(validator != null || isValid == null),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    return child;
  }
}

class ScaffoldPageIcon extends StatelessWidget {
  final Widget icon;
  final Text title;
  final VoidCallback onTap;
  final bool isSelected;

  const ScaffoldPageIcon({
    Key key,
    this.icon,
    this.title,
    this.onTap,
    this.isSelected = false,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final TextPainter textPainter = TextPainter(
        text: TextSpan(
            text: title.data,
            style: TextStyles.textStyleSize(13).copyWith(
                color: CleanAndroidDesignDefaultColors.black,
                fontWeight: FontWeight.w600)),
        maxLines: 1,
        textDirection: TextDirection.ltr)
      ..layout(minWidth: 0, maxWidth: double.infinity);

    return Container(
      width: max(textPainter.size.width, 50) + 10.0,
      child: Column(
        children: <Widget>[
          Container(
            decoration: BoxDecoration(
              boxShadow: isSelected
                  ? [
                      BoxShadow(
                        color: CleanAndroidDesignDefaultColors.defaultColor
                            .withOpacity(0.3),
                        spreadRadius: 0.5,
                        blurRadius: 3,
                        offset: Offset(0, 1),
                      ),
                    ]
                  : [],
              borderRadius: BorderRadius.circular(50),
            ),
            child: Material(
              color: isSelected
                  ? Colors.grey[350]
                  : CleanAndroidDesignDefaultColors.defaultColorLight,
              borderRadius: BorderRadius.circular(50),
              child: InkWell(
                borderRadius: BorderRadius.circular(50),
                onTap: onTap,
                child: Container(
                  width: 50,
                  height: 50,
                  decoration: BoxDecoration(
                    borderRadius: BorderRadius.circular(50),
                  ),
                  child: icon,
                ),
              ),
            ),
          ),
          SizedBox(
            height: 7,
          ),
          GestureDetector(
            onTap: onTap,
            child: DefaultTextStyle(
                style: TextStyles.textStyleSize(13).copyWith(
                    color: CleanAndroidDesignDefaultColors.black,
                    fontWeight: isSelected ? FontWeight.w600 : FontWeight.w400),
                child: title),
          )
        ],
      ),
    );
  }

  ScaffoldPageIcon copyWith({VoidCallback onTap, bool isSelected}) {
    return ScaffoldPageIcon(
      icon: this.icon,
      title: this.title,
      onTap: onTap,
      isSelected: isSelected,
    );
  }
}
