import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

class HighlightBuilder extends StatefulWidget {
  final Widget Function(BuildContext context, Color color) builder;

  final HighlightController controller;

  final Color defaultColor;
  final Color highlightColor;

  const HighlightBuilder({
    Key key,
    this.defaultColor,
    this.highlightColor,
    @required this.controller,
    @required this.builder,
  }) : super(key: key);

  @override
  _HighlightBuilderState createState() => _HighlightBuilderState();
}

class _HighlightBuilderState extends State<HighlightBuilder>
    with SingleTickerProviderStateMixin {
  AnimationController _animationController;
  Animation _colorTween;

  VoidCallback listener;

  @override
  void initState() {
    super.initState();

    listener = () => _animationController.forward();
    widget.controller.addListener(listener);

    _animationController =
        AnimationController(vsync: this, duration: Duration(milliseconds: 300));
    _colorTween = ColorTween(
      begin: widget.defaultColor ?? Colors.grey[300],
      end: widget.highlightColor ?? CleanAndroidDesignDefaultColors.highlight,
    ).animate(_animationController);
    _animationController.addStatusListener((status) {
      if (status == AnimationStatus.completed) _animationController.reverse();
    });
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
        animation: _colorTween,
        builder: (context, _) => widget.builder(context, _colorTween.value));
  }

  @override
  void dispose() {
    super.dispose();

    _animationController.dispose();
    widget.controller.removeListener(listener);
  }
}

class HighlightController implements Listenable {
  ObserverList<VoidCallback> _listeners = ObserverList<VoidCallback>();

  void highlight({Duration delay}) {
    if (delay != null)
      Future.delayed(delay, () => _listeners.forEach((listener) => listener()));
    else
      _listeners.forEach((listener) => listener());
  }

  @override
  void addListener(VoidCallback listener) {
    _listeners.add(listener);
  }

  @override
  void removeListener(VoidCallback listener) {
    _listeners.remove(listener);
  }
}
